/*
 *
 * Author:  Anders Ellefsrud,  Ifi  - University of Oslo
 *
 */

/*
 * $Header: /local/src/CVSROOT/X11/halo/child.c,v 1.11 1994/01/30 00:02:03 karlo Exp $
 * $Log: child.c,v $
 * Revision 1.11  1994/01/30  00:02:03  karlo
 * Fiksa bug med bruk av minne etter frigjering - fungerer p Linux
 *
 * Revision 1.10  1992/12/15  18:00:19  anders
 * Converted to Xt program
 *
 * Revision 1.9  1992/12/08  09:36:51  anders
 * Diagnosed a large-environment problem + som message cleanup.
 *
 * Revision 1.8  1992/12/07  17:25:38  anders
 * linted
 *
 * Revision 1.7  1992/12/07  17:00:08  anders
 * Read the resources in a sub-process to avoid conflicts when
 * reading resources multiple times
 *
 * Revision 1.6  1992/11/19  12:04:40  anders
 * Ymse debugging
 *
 * Revision 1.5  1992/11/18  18:03:54  anders
 * Implementer reread av ressurser vha SIGHUP
 *
 */

#ifndef lint
static char    *rcsid = "$Id: child.c,v 1.11 1994/01/30 00:02:03 karlo Exp $";

#endif

#include <stdio.h>
#include <string.h>
#include <signal.h>

extern char    *realloc ();
extern char    *malloc ();

extern char    *ProgramName;
extern int      verbose;

typedef struct _childrec {
	struct _childrec *next;
	int             pid;
	char           *name;
}               childrec;

childrec       *childs = 0;

/* Add a child to our list */
void
savechild (pid, name)
	int             pid;
	char           *name;
{
	childrec       *p = childs;

	childs = (childrec *) malloc (sizeof (childrec));
	childs->next = p;
	childs->pid = pid;
	childs->name = name;
#ifdef DEBUG
	malloc_verify ();
#endif
}

/* Remove the child with the given pid */
void
childremove (pid)
	int             pid;
{
	childrec       *p;
	childrec       *q;

#ifdef DEBUG
	malloc_verify ();
#endif
	if (!childs) {
		fprintf (stderr, "%s: childremove: internal error, childs list is empty when it should not be\n",
			 ProgramName);
		return;
	}
	if (childs->pid == pid) {
		p = childs;
		free (childs->name);
		childs = childs->next;
		free ((char *)p);
		return;
	} else {
		for (p = childs; p->next; p = p->next) {
			if (p->next->pid == pid) {
				q = p->next;
				p->next = q->next;
				free (q->name);
				free ((char *)q);
				return;
			}
		}
	}
	fprintf (stderr, "%s: childremove: internal error, failed to locate pid %d in childlist\n",
		 ProgramName, pid);
}

/* Returns the name of the child with the given pid */
char           *
findchild (pid)
	int             pid;
{
	childrec       *p;

	for (p = childs; p; p = p->next)
		if (p->pid == pid)
			return p->name;
	return NULL;
}

char *
getlistofchilds()
{
	int len;
	childrec *p;
	static char *childlist = 0;
	len = 1;
	if (!childlist)
		childlist = malloc (100); /* ...or whatever */
	*childlist = 0;
	for (p = childs; p; p = p->next) {
		len += strlen (p->name)+1;
		childlist = realloc (childlist, len);
		strcat (childlist, p->name);
		strcat (childlist, ";");
	}
	childlist[strlen(childlist)-1] = 0;
	return childlist;
}

/* Returns the pid if a child with the given name is found, */
/* else 0 */
int
namechild (name)
	char           *name;
{
	childrec       *p;

	for (p = childs; p; p = p->next)
		if (strcmp (p->name, name) == 0)
			return p->pid;
	fprintf (stderr, "%s: \"%s\" is currently not running\n", ProgramName, name);
	return 0;

}

/* Returns true if all our childs are dead */
int
nochilds ()
{
	return !childs;
}

/* Kill all remaining children */
void
killallchilds ()
{
	void            killunnamedchilds ();

	killunnamedchilds ((char **)0);	       /* Kill all children */
}

/* Kill all our childern not found in "list" */
void
killunnamedchilds (list)
	char          **list;
{
	childrec       *cp;
	char          **lp;
	int             found;
	int             tokill[1024];
	int            *tkp;

	tkp = tokill;
	for (cp = childs; cp; cp = cp->next) {
		found = 0;
		if (list)
			for (lp = list; *lp; lp++)
				if (strcmp (cp->name, *lp) == 0) {
					found = 1;
					break;
				}
		if (!found)
			*tkp++ = cp->pid;
	}
	*tkp = 0;
	for (tkp = tokill; *tkp; tkp++) {
		if (kill (*tkp, SIGTERM) == 0)
			printf ("kill -TERM %d (%s)\n", *tkp, findchild (*tkp));
		else
			printf ("Failed to send TERM to %d (%s)\n", *tkp, findchild (*tkp));
	}	
	sleep (2);
	for (tkp = tokill; *tkp; tkp++)
		if (kill (*tkp, SIGKILL) == 0)
			printf ("kill -KILL %d (%s)\n", *tkp, findchild (*tkp));
}
